/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formcycle;

import static de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroupTestFactory;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import de.ozgcloud.vorgang.common.grpc.GrpcFormDataTestFactory;
import de.ozgcloud.vorgang.vorgang.GrpcFormField;
import de.ozgcloud.vorgang.vorgang.GrpcSubForm;
import lombok.SneakyThrows;

@ITCase
//@SpringBootTest(properties = {
//		"grpc.client.vorgang-manager-local.address=static://127.0.0.1:9090",
//		"grpc.client.vorgang-manager-local.negotiationType=PLAINTEXT"
//})
//@ActiveProfiles("itcase")
@AutoConfigureMockMvc
class FormDataControllerITCase {

	@Autowired
	private MockMvc mockMvc;

	@MockBean
	private SemantikAdapter semantikAdapter;

	@Nested
	class ReceiveFormData {

		@Test
		@SneakyThrows
		void shouldProcessSuccessful() {
			doPostRequest().andExpect(status().isOk());
		}

		@SneakyThrows
		private ResultActions doPostRequest() {
			return mockMvc.perform(
					multipart("/formData")
							.file(new MockMultipartFile("formData", null, FormDataController.HTTP_TYPE_PROTOBUF, buildTestFormData()))
							.file(IncomingFileTestFactory.asMultipartFile("representations"))
							.file(asMultipartFile("attachments",
									createBuilder().name(IncomingFileGroupTestFactory.VENDOR_ID_XXX + "__" + NAME).build())));
		}
	}

	private byte[] buildTestFormData() {
		return FormDataControllerTest.buildTestFormData(FormCycleFormDataTestFactory.withFormData(
				GrpcFormDataTestFactory.createBuilder()
						.addField(GrpcFormField.newBuilder().setName("firstname").setLabel("Vorname").setValue("Theo").build())
						.addField(GrpcFormField.newBuilder().setName("lastname").setLabel("Nachname").setValue("Test").build())
						.addField(GrpcFormField.newBuilder().setName("Street").setLabel("Straße").setValue("Hwy 5").build())
						.addForm(GrpcSubForm.newBuilder()
								.setTitle("Address").setLabel("Adresse")
								.addField(GrpcFormField.newBuilder().setName("firstname").setLabel("Vorname").setValue("Theo").build())
								.addField(GrpcFormField.newBuilder().setName("lastname").setLabel("Nachname").setValue("Test").build())
								.addField(GrpcFormField.newBuilder().setName("street").setLabel("Straße").setValue("Hwy 5").build())
								.build())
						.build()));
	}

}
